/*
 * Bundesliga is Copyright (c) 1997,1998 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "bl.h"
#include "blrcp.h"

/**
 * Database Handling Routinen
 */

static DmOpenRef BlDB;

static SelectedDBInfo dbInfo[10];

CharPtr Items[10];

Err OpenDatabase(UInt cardNo, LocalID dbID)
{
  UInt          index = 0;
  VoidHand      RecHandle;
  VoidPtr	RecPointer;
  Err		err;
  int		i, RecLen;
  Boolean       onlyJustCreated = false;
   
  if (dbID==0) {
    BlDB = DmOpenDatabaseByTypeCreator('Data', 'tpBL', dmModeReadWrite);
  } else {
    BlDB = DmOpenDatabase(cardNo, dbID, dmModeReadWrite);
  }

  if (!BlDB) {
    if ((err = DmCreateDatabase(0, "Bundesliga 98/99", 'tpBL', 'Data', false)))
      return err;
    BlDB = DmOpenDatabaseByTypeCreator('Data', 'tpBL', dmModeReadWrite);

    RecHandle = DmNewRecord(BlDB, &index, sizeof(prefs));
    DmWrite(MemHandleLock(RecHandle), 0, &prefs, sizeof(prefs));
    MemHandleUnlock(RecHandle);
    DmReleaseRecord(BlDB, index, true);

    index = 1;
    RecHandle = DmNewRecord(BlDB, &index, 760);
    DmSet(MemHandleLock(RecHandle), 0, 760, 255);
    MemHandleUnlock(RecHandle); 
    DmReleaseRecord(BlDB, index, true);

    index = 2;
    RecHandle = DmNewRecord(BlDB, &index, sizeof(programme)+sizeof(team));
    DmReleaseRecord(BlDB, index, true);
    onlyJustCreated = true;
  }

  RecHandle = DmGetRecord(BlDB, 0);
  RecPointer = MemHandleLock(RecHandle);
  MemMove(&prefs, RecPointer, sizeof(prefs));
  MemHandleUnlock(RecHandle);
  DmReleaseRecord(BlDB, 0, true);
  
  RecLen=38;
  if (prefs.Version==1) {
    prefs.NumTeams=18;
    RecLen=34;
  }

  RecHandle = DmGetRecord(BlDB, 1);
  RecPointer = MemHandleLock(RecHandle);
  for (i=0; i<prefs.NumTeams; i++) {
    MemMove(MStatus[i].Erg, RecPointer+i*RecLen, RecLen);
    BerechneMStatus(i, (prefs.NumTeams<<1)-3);
  }
  MemHandleUnlock(RecHandle);
  DmReleaseRecord(BlDB, 1, true);
  
  RecLen=(prefs.Version==1)?306:380;
  if (!onlyJustCreated) {
    RecHandle = DmGetRecord(BlDB, 2);
    RecPointer = MemHandleLock(RecHandle);
    if (prefs.Version==1) {
      for (i=0; i<17; i++) {
	MemMove((VoidPtr)programme+i*20, RecPointer+i*18, 18);
      }
    } else MemMove(programme, RecPointer, RecLen);
    for (i=0; i<prefs.NumTeams; i++) {
      MemMove(team[i], RecPointer+RecLen+sizeof(team[i])*i,
	      sizeof(team[0]));
    }
    MemHandleUnlock(RecHandle);
    DmReleaseRecord(BlDB, 2, true);
  }

  prefs.Version = 2;
  Spieltage = (prefs.NumTeams + (prefs.NumTeams%2) -1);
  return 0;
}

void SaveStatus()
{
  Int i, size;
  VoidPtr p; 
  
  prefs.matchDay = matchDay;
  
  p = MemHandleLock(DmResizeRecord(BlDB, 0, sizeof(prefs)));
  DmWrite(p, 0, &prefs, sizeof(prefs));
  MemPtrUnlock(p);
  DmReleaseRecord(BlDB, 0, true);
  
  p = MemHandleLock(DmResizeRecord(BlDB, 1, 20*38));
  for (i=0; i<prefs.NumTeams; i++) {
    DmWrite(p, i*38, MStatus[i].Erg, 38);
  }
  MemPtrUnlock(p);
  DmReleaseRecord(BlDB, 1, true);

  size = sizeof(programme)+sizeof(team);
  if (!DmQueryRecord(BlDB, 2)) {
    UInt index = 2;
    VoidHand RecHandle = DmNewRecord(BlDB, &index, size);
    DmSet(MemHandleLock(RecHandle), 0, i, 255);
    MemHandleUnlock(RecHandle); 
    DmReleaseRecord(BlDB, index, true);
  }

  p = MemHandleLock(DmResizeRecord(BlDB, 2, size));
  DmWrite(p, 0, programme, sizeof(programme));
  DmWrite(p, sizeof(programme), team, sizeof(team));
  MemPtrUnlock(p);
  DmReleaseRecord(BlDB, 2, true);
  DmCloseDatabase(BlDB);
}

static Short FindInstalledDatabases()
{
  Boolean newSearch = true;
  DmSearchStateType stateInfo;
  Short i = 0;

  while (DmGetNextDatabaseByTypeCreator(newSearch, &stateInfo, 'Data', 'tpBL', false, &dbInfo[i].cardNo, &dbInfo[i].dbID) == 0 && i<10) {
    newSearch = false;
    i++;
  }
  return i;
}  

void ChooseDB()
{
  Short nrOfItems, i;
  ListPtr p;
    
  nrOfItems = FindInstalledDatabases();
  for (i=0; i<nrOfItems; i++) {
    DmDatabaseInfo(dbInfo[i].cardNo, dbInfo[i].dbID, dbInfo[i].dbName, 
		   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
    Items[i]=dbInfo[i].dbName;
  }
  p = FrmGetObjectPtr(FrmGetFormPtr(ID_FrmBl),
		      FrmGetObjectIndex(FrmGetFormPtr(ID_FrmBl),
					ID_FrmPopup+1));
  LstSetListChoices(p, Items, nrOfItems);
  LstSetHeight(p, nrOfItems);
}

void SelectedDB(Short i, SelectedDBInfo *info)
{
  MemMove(info, &dbInfo[i], sizeof(SelectedDBInfo));
}
